<?xml version="1.0" encoding="iso-8859-1"?> 
<xsl:stylesheet 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:xsd="http://www.w3.org/2001/XMLSchema"
	exclude-result-prefixes="xsd"
	version="1.0">

<xsl:output 
	method="html" 
	indent="yes" 
	encoding="iso-8859-1"/>

<xsl:strip-space elements="*"/>

	<xsl:template match="/">
		<html>
			<head>
				<style type="text/css">
				table, tr, td, th { border: 1px solid; }
				th                { background-color: yellow; text-align: left; }
				tr                { vertical-align:top; }
				</style>
			</head>
			<body>
				<table>
					<tr>	
						<th>Elment</th>
						<th>Type</th>
						<th>Attribut</th>
						<th>Dfinition</th>
						<th>Commentaire</th>
						<th>Card.</th>
					</tr>
					<xsl:apply-templates select=".//xsd:element"/>
				</table>
			</body>
		</html>
	</xsl:template>

	<!--
	***
	***	Affichage d'un element contenant d'autres elements
	***
	-->
	<xsl:template match="xsd:element[./xsd:element]">
		<tr>
			<th colspan="4">
				<xsl:for-each select="ancestor::xsd:element[./xsd:element]">
					<xsl:variable name="name"><xsl:value-of select="@name"/></xsl:variable>
					<xsl:value-of select="$name"/>
					<xsl:choose>
						<xsl:when test="xsd:attribute [@type='xsd:ID'][@fixed!='']">
							<xsl:text>#</xsl:text>
							<xsl:value-of select="xsd:attribute [@type='xsd:ID']/@fixed"/>
						</xsl:when>
						<xsl:otherwise>
							<xsl:text>[</xsl:text>
							<xsl:value-of select="count(preceding-sibling::xsd:element[@name=$name]) + 1"/>
							<xsl:text>]</xsl:text>
						</xsl:otherwise>
					</xsl:choose>
					<xsl:text>/</xsl:text>
				</xsl:for-each>
				<xsl:variable name="name"><xsl:value-of select="@name"/></xsl:variable>
				<xsl:value-of select="$name"/>
				<xsl:choose>
					<xsl:when test="xsd:attribute [@type='xsd:ID'][@fixed!='']">
						<xsl:text>#</xsl:text>
						<xsl:value-of select="xsd:attribute [@type='xsd:ID']/@fixed"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>[</xsl:text>
						<xsl:value-of select="count(preceding-sibling::xsd:element[@name=$name]) + 1"/>
						<xsl:text>]</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
			</th>
			<th><xsl:apply-templates select="xsd:annotation"/></th>
			<th><xsl:call-template name="cardinalites"/></th>
		</tr>
		<xsl:apply-templates select="xsd:attribute"/>
	</xsl:template>

	<!--
	***
	***	Affichage d'un element
	***
	-->
	<xsl:template match="xsd:element">
		<tr>
			<td><xsl:value-of select="@name"/></td>
			<td><xsl:value-of select="@type"/></td>
			<td/>
			<td><xsl:value-of select="@fixed"/></td>
			<td><xsl:apply-templates select="xsd:annotation"/></td>
			<td><xsl:call-template name="cardinalites"/></td>
		</tr>
		<xsl:apply-templates select="xsd:attribute"/>
	</xsl:template>

	<!--
	***
	***	Affichage d'un attribut
	***
	-->
	<xsl:template match="xsd:attribute[not(@use='prohibited')]">
		<tr>
			<td/>
			<td/>
			<td><xsl:value-of select="@name"/></td>
			<td><xsl:value-of select="@fixed"/></td>
			<td/>
			<td>
				<xsl:choose>
					<xsl:when test="@use = 'required'">
						<xsl:text>[1-1]</xsl:text>
					</xsl:when>
					<xsl:otherwise>
						<xsl:text>[0-1]</xsl:text>
					</xsl:otherwise>
				</xsl:choose>
			</td>
		</tr>
	</xsl:template>

	<!--
	***
	***	Affichage d'une annotation
	***
	-->
	<xsl:template match="xsd:annotation">
		<i><xsl:apply-templates/></i>
	</xsl:template>


	<!--
	***
	***	Affichage des cardinalites d'un element
	***
	-->
	<xsl:template name="cardinalites">
		<xsl:variable name="min">
			<xsl:choose>
				<xsl:when test="not(@minOccurs)">1</xsl:when>
				<xsl:otherwise><xsl:apply-templates select="@minOccurs"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		<xsl:variable name="max">
			<xsl:choose>
				<xsl:when test="not(@maxOccurs)">
					<xsl:choose>
						<xsl:when test="($min = '0')">1</xsl:when>
						<xsl:otherwise><xsl:value-of select="$min"/></xsl:otherwise>
					</xsl:choose>
				</xsl:when>
				<xsl:otherwise><xsl:apply-templates select="@maxOccurs"/></xsl:otherwise>
			</xsl:choose>
		</xsl:variable>
		[<xsl:value-of select="$min"/>-<xsl:value-of select="$max"/>]
	</xsl:template>
	<xsl:template match="@maxOccurs[.='unbounded']">
		<xsl:text>n</xsl:text>
	</xsl:template>
	<xsl:template match="@maxOccurs|@minOccurs">
		<xsl:value-of select="."/>
	</xsl:template>

</xsl:stylesheet>
		