<?php

class ReopenTest extends PastellTestCase
{
    private TypeDossierLoader $typeDossierLoader;

    protected function setUp(): void
    {
        parent::setUp();
        $this->typeDossierLoader = $this->getObjectInstancier()->getInstance(TypeDossierLoader::class);
    }

    protected function tearDown(): void
    {
        parent::tearDown();
        $this->typeDossierLoader->unload();
    }

    private function createDocumentDoubleGed(): string
    {
        $this->typeDossierLoader->createTypeDossierDefinitionFile('double-ged-facultatif');
        $id_ce = $this->createConnector('FakeGED', 'FakeGED')['id_ce'];
        $this->associateFluxWithConnector($id_ce, 'double-ged-facultatif', "GED", 1, 0);
        $this->associateFluxWithConnector($id_ce, 'double-ged-facultatif', "GED", 1, 1);
        $info = $this->createDocument('double-ged-facultatif');
        $id_d = $info['id_d'];
        $this->configureDocument($id_d, ['envoi_depot_1' => 'On', 'envoi_depot_2' => '']);
        return $id_d;
    }

    /**
     * @throws TypeDossierException
     */
    public function testCasNominal(): void
    {
        $id_d = $this->createDocumentDoubleGed();
        $result = $this->triggerActionOnDocument($id_d, 'orientation');
        static::assertTrue($result);
        $this->assertLastDocumentAction('preparation-send-ged_1', $id_d);
        $this->triggerActionOnDocument($id_d, 'send-ged_1');
        $this->assertLastDocumentAction('send-ged_1', $id_d);
        $this->triggerActionOnDocument($id_d, 'orientation');
        $this->assertLastDocumentAction('termine', $id_d);

        $actionExecutorFactory = $this->getObjectInstancier()->getInstance(ActionExecutorFactory::class);
        $actionExecutorFactory->executeLotDocument(1, 1, [$id_d], 'reouverture');
        $jobQueueSQL = $this->getObjectInstancier()->getInstance(JobQueueSQL::class);
        static::assertNotEmpty($jobQueueSQL->getJobIdForDocument(1, $id_d));

        $result = $this->triggerActionOnDocument($id_d, 'reouverture');
        static::assertTrue($result);
        $this->assertLastDocumentAction('send-ged_1', $id_d);
        static::assertEmpty($jobQueueSQL->getJobIdForDocument(1, $id_d));

        $this->configureDocument($id_d, [ 'envoi_depot_2' => true]);
        static::assertEmpty($jobQueueSQL->getJobIdForDocument(1, $id_d));
    }

    /**
     * @throws TypeDossierException
     */
    public function testReopenWhenNotTerminated(): void
    {
        $id_d = $this->createDocumentDoubleGed();
        $result = $this->triggerActionOnDocument($id_d, 'reouverture');
        $this->assertLastMessage(
            "La réouverture du document n'est possible que sur un document dans l'état terminé"
        );
        static::assertFalse($result);
    }
}
